@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -9,6 +9,10 @@
 #ifndef SQUID_ESIPARSER_H
 #define SQUID_ESIPARSER_H
 
+#include "base/RefCount.h"
+
+#include <list>
+
 class ESIParserClient
 {
 public:
@@ -19,8 +23,6 @@ public:
     virtual ~ESIParserClient() {};
 };
 
-#include "base/RefCount.h"
-
 class ESIParser : public RefCountable
 {
 public:
@@ -45,9 +47,7 @@ protected:
 
 private:
     static Register *Parser;
-    static Register *Parsers;
-
-public:
+    static std::list<Register *> & GetRegistry();
 };
 
 class ESIParser::Register
@@ -59,7 +59,6 @@ public:
 
     const char *name;
     ESIParser::Pointer (*newParser)(ESIParserClient *aClient);
-    Register * next;
 };
 
 #define EsiParserDefinition(ThisClass) \
