@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2017 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -18,6 +18,12 @@ namespace Store {
 class Controlled: public Storage
 {
 public:
+    /// \returns a possibly unlocked/unregistered stored entry with key (or nil)
+    /// The returned entry might not match the caller's Store ID or method. The
+    /// caller must abandon()/release() the entry or register it with Root().
+    /// This method must not trigger slow I/O operations (e.g., disk swap in).
+    virtual StoreEntry *get(const cache_key *) = 0;
+
     /// somebody needs this entry (many cache replacement policies need to know)
     virtual void reference(StoreEntry &e) = 0;
 
@@ -28,15 +34,15 @@ public:
     /// make stored metadata and HTTP headers the same as in the given entry
     virtual void updateHeaders(StoreEntry *) {}
 
-    /// If this storage cannot cache collapsed entries, return false.
+    /// If Transients entry cannot be attached to this storage, return false.
     /// If the entry is not found, return false. Otherwise, return true after
-    /// tying the entry to this cache and setting inSync to updateCollapsed().
-    virtual bool anchorCollapsed(StoreEntry &, bool &/*inSync*/) { return false; }
+    /// tying the entry to this cache and setting inSync to updateAnchored().
+    virtual bool anchorToCache(StoreEntry &, bool &/*inSync*/) { return false; }
 
-    /// Update a local collapsed entry with fresh info from this cache (if any).
-    /// Return true iff the cache supports collapsed entries and
-    /// the given local collapsed entry is now in sync with this storage.
-    virtual bool updateCollapsed(StoreEntry &) { return false; }
+    /// Update a local Transients entry with fresh info from this cache (if any).
+    /// Return true iff the cache supports Transients entries and
+    /// the given local Transients entry is now in sync with this storage.
+    virtual bool updateAnchored(StoreEntry &) { return false; }
 };
 
 } // namespace Store
