@@ -28,6 +28,7 @@
 #include "http.h"
 #include "http/Stream.h"
 #include "HttpRequest.h"
+#include "icmp/net_db.h"
 #include "ip/QosConfig.h"
 #include "LogTags.h"
 #include "MemBuf.h"
@@ -1037,6 +1038,8 @@ tunnelConnectDone(const Comm::ConnectionPointer &conn, Comm::Flag status, int xe
         tunnelState->server.setDelayId(DelayId());
 #endif
 
+    netdbPingSite(tunnelState->request->url.host());
+
     tunnelState->request->hier.resetPeerNotes(conn, tunnelState->getHost());
 
     tunnelState->server.conn = conn;
