@@ -45,7 +45,7 @@
  *
  \param argc Ignored.
  \param argv Ignored.
- \retval ERR An error occured removing the file.
+ \retval ERR An error occurred removing the file.
  \retval OK  The file has been removed.
  */
 int
